package {
  import flash.display.Sprite;
  import flash.events.MouseEvent;
  import flash.text.TextField;
  import flash.text.TextFormat;
  
  import net.user1.logger.Logger;
  import net.user1.reactor.AttributeEvent;
  import net.user1.reactor.Reactor;
  import net.user1.reactor.ReactorEvent;
  import net.user1.reactor.Room;
  import net.user1.reactor.RoomEvent;
  import net.user1.reactor.ModuleType;
  import net.user1.reactor.RoomModules;
  import net.user1.reactor.RoomSettings;
  
  public class UnionFridgeMagnets extends Sprite {
    // A hash of magnet name, graphic pairs
    protected var magnets:Object;
    
    // The magnet room
    protected var room:Room;
    
    // The core Reactor object that connects to Union Server
    protected var reactor:Reactor;
    
    // A text field in which to display the number of users
    protected var output:TextField;
    
    // Constructor
    public function UnionFridgeMagnets () {
      // Make the magnets hash
      magnets = new Object();
      
      // Make the output text field
      output = new TextField();
      output.height = 20;
      output.width = stage.stageWidth;
      output.x = 10;
      output.y = 10;
      output.textColor = 0xFFFFFF;
      addChild(output);
      
      // Make the Reactor object and connect to Union Server
      reactor = new Reactor("config.xml");
      reactor.addEventListener(ReactorEvent.READY, readyListener);
    }
    
    // Triggered when the connection is established and ready for use
    protected function readyListener (e:ReactorEvent):void  {
      // Specify the server side room module for the fridge magnet room
      var modules:RoomModules = new RoomModules();
      modules.addModule("net.user1.union.example.roommodule.FridgeMagnetsRoomModule", 
                        ModuleType.CLASS);
      // Keep the fridge magnet room alive forever
      var settings:RoomSettings = new RoomSettings();
      settings.removeOnEmpty = false;
      // Create the fridge magnet room
      room = reactor.getRoomManager().createRoom("examples.fridgemagnets", 
                                                 settings, 
                                                 null, 
                                                 modules);
      // Register to be notified when this client joins the room
      room.addEventListener(RoomEvent.JOIN, joinListener);
      // Register to be notified when the number of room occupants changes
      room.addEventListener(RoomEvent.OCCUPANT_COUNT, occupantCountListener);
      // Registered to be notified when the room's attributes change
      room.addEventListener(AttributeEvent.UPDATE, attributeUpdateListener);
      // Join the room
      room.join();
    }
    
    // Triggered when his client joins the room
    protected function joinListener (e:RoomEvent):void {
      // Get a hash of all room attributes
      var attributes:Object = room.getAttributes();
      var magnetData:Array;
      var magnet:Magnet;
      
      // Initialize the magnets. Each attribute that begins with "magnet"
      // represents a magnet.
      for (var attributeName:String in attributes) {
        // If the attribute name begins with "magnet"...
        if (attributeName.indexOf("magnet") == 0) {
          // Convert the attribute's string value to an array
          magnetData = String(attributes[attributeName]).split(",");
          // Create a new magnet graphic, and add it to the magnets hash
          magnets[attributeName] = new Magnet();
          magnet = magnets[attributeName];
          magnet.name = attributeName;
          magnet.setLabel(magnetData[0]);
          magnet.x = parseInt(magnetData[1]);
          magnet.y = parseInt(magnetData[2]);
          magnet.addEventListener(MouseEvent.MOUSE_DOWN, magnetMouseDownListener);
          magnet.addEventListener(MouseEvent.MOUSE_UP, magnetMouseUpListener);
          addChild(magnet);
        }
      }
    }
    
    // Triggered when one of the room's attributes changes
    protected function attributeUpdateListener (e:AttributeEvent):void {
      var magnetData:Array;
      var magnet:Magnet;
      
      // If the changed attribute's name begins with "magnet"
      if (e.getChangedAttr().name.indexOf("magnet") == 0) {
        // If a magnet by the specified name exists in the magnets hash...
        magnet = magnets[e.getChangedAttr().name];
        if (magnet != null) {
          // The magnet exists. Check if it is being dragged. If not, 
          // move it to the location specified by the room attribute value.
          if (!magnet.isDragging()) {
            magnetData = e.getChangedAttr().value.split(",");
            magnet.x = parseInt(magnetData[1]);
            magnet.y = parseInt(magnetData[2]);
          }
        }
      }
    }
    
    // Triggered when the room occupant count changes
    protected function occupantCountListener (e:RoomEvent):void {
      output.text = "Users connected: " + e.getNumClients();
    }
    
    // Triggered when a magnet is dragged.
    protected function magnetMouseDownListener (e:MouseEvent):void {
      // Move the magnet being dragged in front of all other magnets
      setChildIndex(Magnet(e.target), this.numChildren - 1);
    }
    
    // Triggered when a magnet is released.
    protected function magnetMouseUpListener (e:MouseEvent):void {
      // The magnet was released, so send the moved magnet's new 
      // position to the server. Send the position in a module message
      // rather than assigning the room attribute directly so that
      // the server can decide whether the requested move is legal before
      // allowing the magnet to be repositioned. If the move request is
      // granted, the server will set the room attribute, which will trigger
      // attributeUpdateListener().
      var moduleArgs:Object = new Object();
      moduleArgs.MAGNET = Magnet(e.target).name;
      moduleArgs.X      = Math.floor(Magnet(e.target).x).toString();
      moduleArgs.Y      = Math.floor(Magnet(e.target).y).toString();
      room.sendModuleMessage("MOVE", moduleArgs);
    }
  }
}